@@ -1,8 +1,8 @@
 #!@SH@
 ##
 ##  OSSP shiela - CVS Access Control and Logging Facility
-##  Copyright (c) 2000-2002 Ralf S. Engelschall <rse@engelschall.com>
-##  Copyright (c) 2000-2002 The OSSP Project <http://www.ossp.org/>
+##  Copyright (c) 2000-2004 Ralf S. Engelschall <rse@engelschall.com>
+##  Copyright (c) 2000-2004 The OSSP Project <http://www.ossp.org/>
 ##
 ##  This file is part of OSSP shiela, an access control and logging
 ##  facility for Concurrent Versions System (CVS) repositories
@@ -26,7 +26,7 @@
 ##  shiela-install.sh: repository install program (syntax: Bourne-Shell)
 ##
 
-version="1.0.4"
+version="1.1.0"
 
 prefix="@prefix@"
 bindir="@bindir@"
@@ -104,7 +104,7 @@ not immediately correct.
 EOT
 
 for arg; do
-    eval `echo $arg | sed -e 's/^\([^=]*\)=\(.*\)$/V_\1="\2"/`
+    eval `echo "$arg" | sed -e 's/^\([^=]*\)=\(.*\)$/V_\1="\2"/'`
 done
 
 query () {
@@ -371,10 +371,10 @@ else
     CVS_VENDOR=CVSHome.org
 fi
 case $CVS_VERSION in
-   1.10.[789]* | 1.10.1[0123456789]* | 1.11.* )
+   1.1[2-9]* )
        ;;
    * )
-       echo "ERROR: CVS has to be at least version 1.10.7"
+       echo "ERROR: CVS has to be at least version 1.12"
        exit 1
        ;;
 esac
@@ -411,8 +411,8 @@ query U shiela_local
 cat >$tmpdir/shiela.cfg <<EOT
 ##
 ##  OSSP shiela - Access Control and Logging Facility for CVS
-##  Copyright (c) 2000-2002 Ralf S. Engelschall <rse@engelschall.com>
-##  Copyright (c) 2000-2002 The OSSP Project <http://www.ossp.org/>
+##  Copyright (c) 2000-2004 Ralf S. Engelschall <rse@engelschall.com>
+##  Copyright (c) 2000-2004 The OSSP Project <http://www.ossp.org/>
 ##
 ##  This file is part of OSSP shiela, an access control and logging
 ##  facility for Concurrent Versions System (CVS) repositories
@@ -441,6 +441,7 @@ Environment {
     Program sendmail ${V_tool_sendmail};
     Program cvs      ${V_tool_cvs};
     Program diff     ${V_tool_diff};
+    Program xdelta   ${V_tool_xdelta};
     Program uuencode ${V_tool_uuencode};
 };
 
@@ -477,8 +478,8 @@ Repository {
 Logging {
     Reports {
         Report mail {
-            Content title rule header files log summary details;
-            Details diff:mime;
+            Content title rule header files log summary rule details;
+            Details patch:plain;
         };
     };
 };
@@ -535,9 +536,9 @@ echo ""
 echo "${term_bold}CVSROOT MODIFICATION${term_norm}"
 cat <<EOT
 
-Ok, we are now prepared to modify ${V_repos_path}/CVSROOT/. We will
-first checkout CVSROOT and modify it. Then you have a last chance to
-stop processing before we finally commit the modifications.
+Ok, we are now prepared to modify your repository. We will first
+checkout CVSROOT and modify it. Then you have a last chance to stop
+processing before we finally commit the modifications.
 
 EOT
 
@@ -586,6 +587,14 @@ run cp $tmpdir/shiela.msg .
 run chmod 644 shiela.msg
 cvs_file add shiela.msg
 
+cvs_file tst config
+if [ ".`grep -i UseNewInfoFmtStrings config`" != . ]; then
+    run "cp config config.old && sed -e 's;UseNewInfoFmtStrings=.*;UseNewInfoFmtStrings=yes;' <config.old >config && rm -f config.old"
+else
+    run "(echo ''; echo 'UseNewInfoFmtStrings=yes'; echo '') >>config"
+fi
+cvs_file add config
+
 cvs_file tst checkoutlist
 run "echo 'shiela.cfg' >>checkoutlist"
 run "echo 'shiela' >>checkoutlist"
@@ -597,7 +606,7 @@ run "(echo ''; echo 'ALL \$CVSROOT/CVSROOT/shiela.msg'; echo '') >>rcsinfo"
 cvs_file add rcsinfo
 
 cvs_file tst taginfo
-run "(echo ''; echo 'ALL $V_tool_shiela --hook=taginfo'; echo '') >>taginfo"
+run "(echo ''; echo 'ALL $V_tool_shiela --hook=taginfo %t %o %p %{sv}'; echo '') >>taginfo"
 cvs_file add taginfo
 
 if [ ".$CVS_VENDOR" = .RSE ]; then
@@ -611,11 +620,11 @@ if [ ".$CVS_VENDOR" = .RSE ]; then
 fi
 
 cvs_file tst commitinfo
-run "(echo ''; echo 'ALL $V_tool_shiela --hook=commitinfo'; echo '') >>commitinfo"
+run "(echo ''; echo 'ALL $V_tool_shiela --hook=commitinfo %r/%p %s'; echo '') >>commitinfo"
 cvs_file add commitinfo
 
 cvs_file tst verifymsg
-run "(echo ''; echo 'DEFAULT $V_tool_shiela --hook=verifymsg'; echo '') >>verifymsg"
+run "(echo ''; echo 'DEFAULT $V_tool_shiela --hook=verifymsg %l'; echo '') >>verifymsg"
 cvs_file add verifymsg
 
 if [ ".$CVS_VENDOR" = .RSE ]; then
@@ -624,7 +633,7 @@ else
     flags="sVv"
 fi
 cvs_file tst loginfo
-run "(echo ''; echo 'ALL $V_tool_shiela --hook=loginfo %{${flags}}'; echo '') >>loginfo"
+run "(echo ''; echo 'ALL $V_tool_shiela --hook=loginfo %p %{${flags}}'; echo '') >>loginfo"
 cvs_file add loginfo
 
 run $V_tool_cvs update
@@ -653,11 +662,11 @@ run rm -rf CVSROOT
 
 cat <<EOT
 
-Ok, your $V_repos_path/CVSROOT/ area was modified to now use OSSP shiela
-for access control and logging. You can try it out by checking out
-CVSROOT yourself and modify some files. You should receive an Email
-for every modification. Then especially adjust "shiela.cfg" for your
-particular repository layout.
+Ok, your CVS repository was modified to now use OSSP shiela for access
+control and logging. You can try it out by checking out CVSROOT
+yourself and modify some files. You should receive an Email for every
+modification. Then especially adjust "shiela.cfg" for your particular
+repository layout.
 
 EOT
 
