@@ -1,8 +1,8 @@
 #!@PERL@ -w
 ##
 ##  OSSP shiela - CVS Access Control and Logging Facility
-##  Copyright (c) 2000-2002 Ralf S. Engelschall <rse@engelschall.com>
-##  Copyright (c) 2000-2002 The OSSP Project <http://www.ossp.org/>
+##  Copyright (c) 2000-2004 Ralf S. Engelschall <rse@engelschall.com>
+##  Copyright (c) 2000-2004 The OSSP Project <http://www.ossp.org/>
 ##
 ##  This file is part of OSSP shiela, an access control and logging
 ##  facility for Concurrent Versions System (CVS) repositories
@@ -26,7 +26,7 @@
 ##  shiela.pl: control program (syntax: Perl)
 ##
 
-my $version = '1.0.4';
+my $version = '1.1.0';
 
 require 5.005;
 
@@ -212,7 +212,7 @@ sub rt_determine_two {
     $RT->{cvsvers} = $1 if ($v =~ m|Concurrent\s+Versions\s+System\s+\(CVS\)\s+([\d.p]+)\s+|s);
     $RT->{cvsrse} = 0;
     $RT->{cvsrse} = 1 if ($v =~ m|\[RSE\]|s);
-    die "$RT->{cvs} is not at least CVS 1.10" if ($RT->{cvsvers} !~ m|^1\.1[0-9]|);
+    die "$RT->{cvs} is not at least CVS 1.12" if ($RT->{cvsvers} !~ m|^1\.1[2-9]|);
     $RT->{useserver} = 0;
     $RT->{useserver} = 1 if ($v =~ m|server|s);
 
@@ -580,7 +580,7 @@ sub new {
     STDERR->flush; # because of fork() behind open2()!
     $cvs->{rfd} = new IO::Handle;
     $cvs->{wfd} = new IO::Handle;
-    $cvs->{pid} = IPC::Open2::open2($cvs->{rfd}, $cvs->{wfd}, "$program -Q -l -n server")
+    $cvs->{pid} = IPC::Open2::open2($cvs->{rfd}, $cvs->{wfd}, "$program -f -Q -n server")
         || die "cannot spawn CVS server process `$program server'";
     print STDERR "cvs server: spawned (pid $cvs->{pid})\n" if ($trace);
     bless ($cvs, $class);
@@ -641,7 +641,7 @@ sub send {
 sub recv {
     my $cvs = shift;
     if (wantarray) {
-         my @lines = $cvs->{rfd}->getlines;
+         my @lines = ($cvs->{rfd}->getlines || ());
          my @nlines = ();
          foreach my $line (@lines) {
              print STDERR "cvs server: <- $line" if ($cvs->{trace});
@@ -651,7 +651,7 @@ sub recv {
          return @nlines;
     }
     else {
-         my $line = $cvs->{rfd}->getline;
+         my $line = ($cvs->{rfd}->getline || "");
          print STDERR "cvs server: <- $line" if ($cvs->{trace});
          $line =~ s|\n$||;
          return $line;
@@ -1248,14 +1248,14 @@ sub hook_commitinfo {
     #   annotate the files with the branch they stay on
     my $cvsstat = '';
     if (not $RT->{useserver}) {
-        my $io = new IO::File "$RT->{cvs} -f -l -Q -n status ".join(' ', @cvsfiles)."|"
+        my $io = new IO::File "$RT->{cvs} -f -Q -n status ".join(' ', @cvsfiles)."|"
             || die "unable to open CVS command pipe for reading";
         $cvsstat .= $_ while (<$io>);
         $io->close;
     }
     else {
         my $cvs = new CVS ($RT->{cvs}, $RT->{cvsroot});
-        $cvs->global_options("-l", "-Q", "-n");
+        $cvs->global_options("-Q", "-n");
         $cvs->directory($cvsdir);
         foreach my $cvsfile (@cvsfiles) {
             $cvs->entry($cvsfile);
@@ -1425,7 +1425,12 @@ sub hook_loginfo_collect {
     my ($PA, $RT, $CF) = @_;
 
     #   take the arguments
-    my ($cvsdir, @cvsinfo) = split(/\s+/, $PA->{ARG}->[0]);
+    my $cvsdir = $PA->{ARG}->[0];
+    my @cvsinfo = ();
+    my $k = ($RT->{cvsrse} ? 5 : 3);
+    for (my $i = 1; $i <= $#{$PA->{ARG}}; $i += $k) {
+        push(@cvsinfo, join(",", @{$PA->{ARG}}[$i..$i+$k-1]));
+    }
 
     #   suck in the standard log information which CVS provides
     my $cvsmsg = '';
@@ -1435,7 +1440,7 @@ sub hook_loginfo_collect {
     $RT->{cvsop} = 'commit-file';
 
     #   handle special invocation under `cvs add <dir>'
-    if (join(' ', @cvsinfo) eq '- New directory') { # see CVS' src/add.c
+    if ($PA->{ARG}->[1] eq '- New directory') { # see CVS' src/add.c
         #   Hmmm... we always just deal with files in OSSP shiela, so there
         #   is no obvious and consistent way to deal now with only a
         #   plain directory. And there is also no log message provided
@@ -1451,7 +1456,7 @@ sub hook_loginfo_collect {
     #   CVS only calls us inside the loginfo hook and never in the
     #   commitinfo hook before. Additionally CVS doesn't provide us with
     #   the %{sVvto} information :(
-    if (join(' ', @cvsinfo) eq '- Imported sources') { # see CVS' src/import.c
+    if ($PA->{ARG}->[1] eq '- Imported sources') { # see CVS' src/import.c
         #   I = ignored
         #   L = link (=error),
         #   N = new file
@@ -1488,7 +1493,7 @@ sub hook_loginfo_collect {
                     print STDERR "cvs import: Ignoring this operation - don't expect log messages!\n";
                     exit(0);
                 }
-                my $io = new IO::File "$RT->{cvs} -f -l -Q -n log -r$It '$Is'|"
+                my $io = new IO::File "$RT->{cvs} -f -Q -n log -r$It '$Is'|"
                     || die "unable to open CVS command pipe for reading";
                 $rcslog = $_ while (<$io>);
                 $io->close;
@@ -1610,7 +1615,7 @@ sub hook_loginfo_collect {
         if ($Io eq 'R' and $Iv eq 'NONE') {
             my $rcslog ='';
             if (not $RT->{useserver}) {
-                my $io = new IO::File "$RT->{cvs} -f -l -Q -n log '$Is'|"
+                my $io = new IO::File "$RT->{cvs} -f -Q -n log '$Is'|"
                     || die "unable to open CVS command pipe for reading";
                 $rcslog .= $_ while (<$io>);
                 $io->close;
@@ -1633,7 +1638,7 @@ sub hook_loginfo_collect {
         my $rcslog = '';
         if ($Io eq 'A' or $Io eq 'M' or $Io eq 'R') {
             if (not $RT->{useserver}) {
-                my $io = new IO::File "$RT->{cvs} -f -l -Q -n log -r$Iv '$Is'|"
+                my $io = new IO::File "$RT->{cvs} -f -Q -n log -r$Iv '$Is'|"
                     || die "unable to open CVS command pipe for reading";
                 $rcslog .= $_ while (<$io>);
                 $io->close;
@@ -1669,7 +1674,7 @@ sub hook_loginfo_collect {
         }
         else {
             if ($Io eq 'A') {
-                my $io = new IO::File "<$Is" || die "unable open $Is for reading";
+                my $io = new IO::File "<$Is" || die "unable to open $Is for reading";
                 my $l = 0;
                 $l++ while (<$io>);
                 $io->close;
@@ -1706,7 +1711,7 @@ sub hook_loginfo_collect {
             my $io = new IO::File ">$RT->{tmpfile}.all"
                 || die "unable to open temporary file $RT->{tmpfile}.all for writing";
             if (not $RT->{useserver}) {
-                my $cvs = new IO::File "$RT->{cvs} -f -l -Q -n update -p -r$Iv '$Is'|"
+                my $cvs = new IO::File "$RT->{cvs} -f -Q -n update -p -r$Iv '$Is'|"
                     || die "unable to open CVS command pipe for reading";
                 $io->print($_) while (<$cvs>);
                 $cvs->close;
@@ -1789,7 +1794,7 @@ sub hook_loginfo_collect {
                     my $io = new IO::File ">$RT->{tmpfile}.old"
                         || die "unable to open temporary file $RT->{tmpfile}.old for writing";
                     if (not $RT->{useserver}) {
-                        my $cvs = new IO::File "$RT->{cvs} -f -l -Q -n update -p -r$IV '$Is'|"
+                        my $cvs = new IO::File "$RT->{cvs} -f -Q -n update -p -r$IV '$Is'|"
                             || die "unable to open CVS command pipe for reading";
                         $io->print($_) while (<$cvs>);
                         $cvs->close;
@@ -1810,7 +1815,7 @@ sub hook_loginfo_collect {
                     $io = new IO::File ">$RT->{tmpfile}.new"
                         || die "unable to open temporary file $RT->{tmpfile}.new for writing";
                     if (not $RT->{useserver}) {
-                        my $cvs = new IO::File "$RT->{cvs} -f -l -Q -n update -p -r$Iv '$Is'|"
+                        my $cvs = new IO::File "$RT->{cvs} -f -Q -n update -p -r$Iv '$Is'|"
                             || die "unable to open CVS command pipe for reading";
                         $io->print($_) while (<$cvs>);
                         $cvs->close;
@@ -1856,7 +1861,7 @@ sub hook_loginfo_collect {
                 #   generate textual change patch script
                 my $d = '';
                 if (not $RT->{useserver}) {
-                    my $io = new IO::File "$RT->{cvs} -f -l -Q -n diff -u -r$IV -r$Iv '$Is'|"
+                    my $io = new IO::File "$RT->{cvs} -f -Q -n diff -u -r$IV -r$Iv '$Is'|"
                         || die "unable to open CVS command pipe for reading";
                     $d .= $_ while (<$io>);
                     $io->close;
@@ -2245,6 +2250,7 @@ sub produce_log_message {
         elsif ($content eq 'details') {
             $O .= "\n";
             if ($style =~ m|^url:(.+)|) {
+                $O .= "Change details:\n";
                 my $urlspec = $1;
                 foreach $file (sort(keys(%{$IN->{file}}))) {
                     next if ($IN->{file}->{$file}->{op} eq 'T');
@@ -2254,7 +2260,26 @@ sub produce_log_message {
                         elsif ($1 eq 'V') { $IN->{file}->{$file}->{oldrev}; }
                         elsif ($1 eq 'v') { $IN->{file}->{$file}->{newrev}; }
                     |gse;
-                    $O .= "$prefix$url\n";
+                    $O .= "  $prefix$url\n";
+                }
+            }
+            elsif ($style eq 'rdiff') {
+                $O .= "Change details:\n";
+                foreach $file (sort(keys(%{$IN->{file}}))) {
+                    next if ($IN->{file}->{$file}->{op} eq 'T');
+                    if ($IN->{file}->{$file}->{op} eq 'A') {
+                        $O .= "  \$ cvs rdiff -u" .
+                              " -r0 -r" . $IN->{file}->{$file}->{newrev} .
+                              " " . $file .
+                              "\n";
+                    }
+                    else {
+                        $O .= "  \$ cvs rdiff -u" .
+                              " -r" . $IN->{file}->{$file}->{oldrev} .
+                              " -r" . $IN->{file}->{$file}->{newrev} .
+                              " " . $file .
+                              "\n";
+                    }
                 }
             }
             elsif ($style eq 'patch:plain') {
